import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:osam/osam.dart';
import 'package:osam_flutter/osam_flutter.dart';

mixin NavigationPresenter<US> on Presenter<US> {
  NavigationState get navigationState;

  StreamSubscription<List<String>> routesSub;

  @override
  @mustCallSuper
  void init() => routesSub = navigationState.routesStream.listen((routes) {});

  ValueStream<List<String>> get routesStream => navigationState.routesStream;

  void pop() => navigationState.pop();

  void push(String route) => navigationState.push(route);

  void pushReplacement(String route) => navigationState.pushReplacement(route);

  void popUntil(String route) => navigationState.popUntil(route);

  void pushAndRemoveUntil(String route) => navigationState.pushAndRemoveUntil(route);

  List<String> get currentRoutes => routesStream.value;

  String get currentRoute => currentRoutes.last;

  bool isInStack(String route) => currentRoutes.contains(route);

  bool get canPop => navigationState.routesStream.value.length > 1;

  @override
  @mustCallSuper
  void dispose() {
    routesSub.cancel();
    super.dispose();
  }
}

mixin TabNavigationPresenter<US> on Presenter<US> {
  TabNavigationState get tabNavigationState;

  void changeTab(String tab) => tabNavigationState.update((state) => state.tab = tab);

  ValueStream<String> get tabStream => tabNavigationState.propertyStream((state) => state.tab);
}
