import 'package:flutter/widgets.dart';
import 'package:osam/osam.dart';
import 'package:provider/provider.dart';
import 'package:provider/single_child_widget.dart';

part 'presenter.dart';

class UseCaseProvider extends InheritedWidget {
  const UseCaseProvider(
    this._appState,
    this._useCaseContainer,
    this._uiState, {
    Key key,
    Widget child,
    this.persist,
  }) : super(key: key, child: child);

  final Object _appState;
  final UseCaseContainer _useCaseContainer;
  final Object _uiState;
  final Persist<Object, Object> persist;

  static UseCaseProvider of(BuildContext context) =>
      context.dependOnInheritedWidgetOfExactType(aspect: UseCaseProvider);

  @override
  bool updateShouldNotify(UseCaseProvider oldWidget) => false;
}

class PersistLifecycleWrapper extends StatefulWidget {
  final Widget child;

  const PersistLifecycleWrapper({Key key, this.child}) : super(key: key);

  @override
  _PersistLifecycleWrapperState createState() => _PersistLifecycleWrapperState();
}

class _PersistLifecycleWrapperState extends State<PersistLifecycleWrapper> with WidgetsBindingObserver {
  @override
  void initState() {
    WidgetsBinding.instance.addObserver(this);
    super.initState();
  }

  @override
  Widget build(BuildContext context) => widget.child;

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.paused) {
      final useCaseProvider = UseCaseProvider.of(context);
      useCaseProvider.persist.save(useCaseProvider._appState, useCaseProvider._uiState);
    }
  }
}

mixin _PresenterProviderSingleChildWidget on SingleChildWidget {}

class PresenterProvider<P extends Presenter> extends SingleChildStatelessWidget
    with _PresenterProviderSingleChildWidget {
  final Widget child;
  final P presenter;

  const PresenterProvider({Key key, this.child, this.presenter}) : super(key: key, child: child);

  static P of<P extends Presenter>(BuildContext context) => Provider.of<P>(context, listen: false);

  @override
  Widget buildWithChild(BuildContext context, Widget child) {
    final useCaseProvider = UseCaseProvider.of(context);
    return Provider<P>(
      create: (_) {
        presenter?._useCaseContainer = useCaseProvider._useCaseContainer;
        presenter?._uiState = useCaseProvider?._uiState;
        presenter?.init();
        return presenter;
      },
      child: child,
      dispose: (ctx, presenter) => presenter?.dispose(),
    );
  }
}

class MultiPresenterProvider extends StatelessWidget {
  final List<SingleChildWidget> providers;
  final Widget child;

  const MultiPresenterProvider({Key key, @required this.providers, @required this.child}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: providers,
      key: ValueKey('PresenterMultiProvider: %${key.toString()}'),
      child: child,
    );
  }
}

extension PresenterProviderExtension on BuildContext {
  P presenter<P extends Presenter>() => PresenterProvider.of<P>(this);
}
